# exp.r 
# source("exp.r")
# Creates Figure 3 in "Efficient Shrinkage in Parametric Models"
# This is an R program file

rep = 10000
cn  = 50	# number of gridpoints
cm  = 2		# largest value of c for display
nn  = 5		# number of estimators

nlist=c(10,40)
ns=length(nlist)

plist=c(10,40)
ps=length(plist)

risk1=matrix(0,cn,ps*ns)
risk2=matrix(0,cn,ps*ns)
risk3=matrix(0,cn,ps*ns)
risk4=matrix(0,cn,ps*ns)
cc=(0:(cn-1))*(cm/(cn-1))

for (ki in 1:ps){
p=plist[ki]
k=p+1
R = diag(1,k,p)-rbind(matrix(0,1,p),diag(1,p,p))

for (ni in 1:ns) {
n=nlist[ni]

for (ci in 1:cn) {

c=cc[ci];
beta=(0:(k-1))*(c/(k-1))+1
mse=matrix(0,rep,nn)

for (i in 1:rep) {

y=matrix(0,n,k)
for (j in 1:k)  y[,j]=rexp(n,1/beta[j])

# Restricted MLE
btilde=mean(y)
l0 = -n*k*(log(btilde)+1)
btilde=matrix(btilde,k,1)

# Unrestricted MLE
bhat=colMeans(y)
vhat=diag(bhat^2)
bhat=as.matrix(bhat)
l1 = -n*sum(log(bhat))-n*k

# Pre-Test Estimate
lr = 2*(l1-l0)
if (pchisq(lr,p) > 0.95) blr=bhat else blr=btilde

# Shrinkage Estimate
db = bhat-btilde
dn = n*(t(db)%*%db)
A = (vhat%*%R)%*%solve(t(R)%*%vhat%*%R) %*% (t(R)%*%vhat)
tau=sum(diag(A))-2*eigen(A)$value[1]
w=1-tau/dn
if (w<0) w=0
bshrink = bhat%*%w+btilde%*%(1-w)

# Weighted AIC 
w = 1/(1+exp(p-lr/2))
baic = bhat%*%w+btilde%*%(1-w)

# Weighted BIC
w = 1/(1+exp(p*log(n)/2-lr/2))
bbic = bhat%*%w+btilde%*%(1-w)

mse[i,1]=sum((bhat-beta)^2)
mse[i,2]=sum((bshrink-beta)^2)
mse[i,3]=sum((blr-beta)^2)
mse[i,4]=sum((baic-beta)^2)
mse[i,5]=sum((bbic-beta)^2)

}
r=colMeans(mse)
risk1[ci,2*(ki-1)+ni]=r[2]/r[1]
risk2[ci,2*(ki-1)+ni]=r[3]/r[1]
risk3[ci,2*(ki-1)+ni]=r[4]/r[1]
risk4[ci,2*(ki-1)+ni]=r[5]/r[1]

}
}
}

windows()

rmle=matrix(1,cn,1)
par(mfrow=c(2,2))
ltype=c(1,5,2,4,3)
lthick=c(1,1,1,1,2)

rr=cbind(risk1[,1],risk2[,1],risk3[,1],risk4[,1],rmle)
matplot(cc,rr,type="l",main=paste("n=",nlist[1],", p=",plist[1]),xlab="c",ylab="MSE",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.4))

rr=cbind(risk1[,2],risk2[,2],risk3[,2],risk4[,2],rmle)
matplot(cc,rr,type="l",main=paste("n=",nlist[2],", p=",plist[1]),xlab="c",ylab="MSE",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.4))

rr=cbind(risk1[,3],risk2[,3],risk3[,3],risk4[,3],rmle)
matplot(cc,rr,type="l",main=paste("n=",nlist[1],", p=",plist[2]),xlab="c",ylab="MSE",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.4))

rr=cbind(risk1[,4],risk2[,4],risk3[,4],risk4[,4],rmle)
matplot(cc,rr,type="l",main=paste("n=",nlist[2],", p=",plist[2]),xlab="c",ylab="MSE",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.4))

legend("bottomright",legend=c("Shrinkage","PreTest","W-AIC","BMA","MLE"),lwd=lthick,lty=ltype,bty="n")

