# probit.r 
# source("probit.r")
# Creates Figure 2 in "Efficient Shrinkage in Parametric Models"
# This is an R program file

rep = 10000
rho = .5
b   = 0		
cn  = 50	# number of gridpoints
k   = 4
cm  = .2	# largest value of c for Figure 2 in paper
# cm  = .5	# largest value of c for robustness check
nn  = 8		# number of estimators

nlist=c(200,500)
ns=length(nlist)

plist=c(4,8)
ps=length(plist)

riskb = array(0,dim=c(nn,ps,ns,cn))
riskp = array(0,dim=c(nn,ps,ns,cn))

for (ki in 1:ps){
p=plist[ki]

kk=k+p
v=matrix(rho,kk,kk)
diag(v)=1
vc=chol(v)

cc=(0:(cn-1))*(cm/(cn-1))

for (ni in 1:ns) {
n=nlist[ni]

for (ci in 1:cn) {

c=cc[ci];
theta1 = matrix(b,k,1)
theta2 = matrix(c,p,1)
theta = rbind(0,theta1,theta2)
beta0 = theta[2:(kk+1)]*dnorm(theta[1])
pi0 = pnorm(theta[1]+theta[2:(kk+1)]*2)-pnorm(theta[1])

mseb = matrix(0,rep,nn)
msep = matrix(0,rep,nn)

for (i in 1:rep) {

x  = matrix(rnorm(kk*n),n,kk)%*%vc
x1 = x[,1:k]
x2 = x[,(k+1):kk]
ys = x1%*%theta1 + x2%*%theta2 + rnorm(n)
y  = ifelse(ys<=0,0,1)

# Restricted MLE
probit0.out <- (glm(y~x1, family=binomial(link="probit"),options(warn=-1)))
l0=deviance(probit0.out)
thetatilde=as.matrix(coef(probit0.out))
thetatilde=rbind(thetatilde,matrix(0,p,1))
betatilde = as.matrix(thetatilde[2:(kk+1)]*dnorm(thetatilde[1]))
pitilde = as.matrix(pnorm(thetatilde[1]+thetatilde[2:(kk+1)]*2)-pnorm(thetatilde[1]))

# Unrestricted MLE
probit1.out <- (glm(y~x, family=binomial(link="probit"),options(warn=-1)))
thetahat=as.matrix(coef(probit1.out))
vtheta=vcov(probit1.out)
l1=deviance(probit1.out)
test = (probit1.out$converged==FALSE)
if (test == 1) thetahat=thetatilde
betahat = as.matrix(thetahat[2:(kk+1)]*dnorm(thetahat[1]))
pihat = as.matrix(pnorm(thetahat[1]+thetahat[2:(kk+1)]*2)-pnorm(thetahat[1]))

# Pre-Test Estimate
lr = l0-l1
if (pchisq(lr,p) > 0.95) blr=betahat else blr=betatilde
if (pchisq(lr,p) > 0.95) pilr=pihat else pilr=pitilde

# Weighted AIC 
w = 1/(1+exp(p-lr/2))
baic = betahat%*%w+betatilde%*%(1-w)
piaic = pihat%*%w+pitilde%*%(1-w)

# Weighted BIC
w = 1/(1+exp(p*log(n)/2-lr/2))
bbic = betahat%*%w+betatilde%*%(1-w)
pibic = pihat%*%w+pitilde%*%(1-w)

# Shrinkage Estimators
R = rbind(matrix(0,k+1,p),diag(p))

d1 = t(thetahat-thetatilde)%*%solve(vtheta)%*%(thetahat-thetatilde)
w1 = 1 - (p-2)/d1
if (w1<0) w1=0
bshrink1 = betahat%*%w1+betatilde%*%(1-w1)
pishrink1 = pihat%*%w1+pitilde%*%(1-w1)

A2 = (vtheta%*%R)%*%solve(t(R)%*%vtheta%*%R)%*%(t(R)%*%vtheta)
tau2 = sum(diag(A2)) - 2*eigen(A2)$value[1]
d2 = crossprod(thetahat-thetatilde)
w2 = 1 - tau2/d2
if (w2<0) w2=0
bshrink2 = betahat%*%w2+betatilde%*%(1-w2)
pishrink2 = pihat%*%w2+pitilde%*%(1-w2)

G3 = rbind(-t(betahat*thetahat[1]),diag(dnorm(thetahat[1]),kk))
A3 = (t(G3)%*%vtheta%*%R)%*%solve(t(R)%*%vtheta%*%R)%*%(t(R)%*%vtheta%*%G3)
tau3 = sum(diag(A3)) - 2*eigen(A3)$value[1]
d3 = crossprod(betahat-betatilde)
w3 = 1 - tau3/d3
if (w3<0) w3=0
bshrink3 = betahat%*%w3+betatilde%*%(1-w3)
pishrink3 = pihat%*%w3+pitilde%*%(1-w3)

G4 = rbind(dnorm(2*t(thetahat[2:(kk+1)])+thetahat[1])-dnorm(thetahat[1]),diag(2*dnorm(2*thetahat[2:(kk+1)]+thetahat[1])))
A4 = (t(G4)%*%vtheta%*%R)%*%solve(t(R)%*%vtheta%*%R)%*%(t(R)%*%vtheta%*%G4)
tau4 = sum(diag(A4)) - 2*eigen(A4)$value[1]
d4 = crossprod(pihat-pitilde)
w4 = 1 - tau4/d4
if (w4<0) w4=0
bshrink4 = betahat%*%w4+betatilde%*%(1-w4)
pishrink4 = pihat%*%w4+pitilde%*%(1-w4)

mseb[i,1]=sum((betahat-beta0)^2)
mseb[i,2]=sum((blr-beta0)^2)
mseb[i,3]=sum((baic-beta0)^2)
mseb[i,4]=sum((bbic-beta0)^2)
mseb[i,5]=sum((bshrink1-beta0)^2)
mseb[i,6]=sum((bshrink2-beta0)^2)
mseb[i,7]=sum((bshrink3-beta0)^2)
mseb[i,8]=sum((bshrink4-beta0)^2)

msep[i,1]=sum((pihat-pi0)^2)
msep[i,2]=sum((pilr-pi0)^2)
msep[i,3]=sum((piaic-pi0)^2)
msep[i,4]=sum((pibic-pi0)^2)
msep[i,5]=sum((pishrink1-pi0)^2)
msep[i,6]=sum((pishrink2-pi0)^2)
msep[i,7]=sum((pishrink3-pi0)^2)
msep[i,8]=sum((pishrink4-pi0)^2)

}
r=colMeans(mseb)
riskb[,ki,ni,ci] = r/r[1]

r=colMeans(msep)
riskp[,ki,ni,ci] = r/r[1]

}
}
}

windows()

par(mfrow=c(2,2))
ltype=c(1,5,2,4,3)
lthick=c(1,1,2)

rr=cbind(riskb[5,1,1,],riskb[2,1,1,],riskb[3,1,1,],riskb[4,1,1,],riskb[1,1,1,])
matplot(cc,rr,type="l",main=paste("n=",nlist[1],", p=",plist[1]),xlab="c",ylab="MSE(beta)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

rr=cbind(riskb[5,1,2,],riskb[2,1,2,],riskb[3,1,2,],riskb[4,1,2,],riskb[1,1,2,])
matplot(cc,rr,type="l",main=paste("n=",nlist[2],", p=",plist[1]),xlab="c",ylab="MSE(beta)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

rr=cbind(riskb[5,2,1,],riskb[2,2,1,],riskb[3,2,1,],riskb[4,2,1,],riskb[1,2,1,])
matplot(cc,rr,type="l",main=paste("n=",nlist[1],", p=",plist[2]),xlab="c",ylab="MSE(beta)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

rr=cbind(riskb[5,2,2,],riskb[2,2,2,],riskb[3,2,2,],riskb[4,2,2,],riskb[1,2,2,])
matplot(cc,rr,type="l",main=paste("n=",nlist[2],", p=",plist[2]),xlab="c",ylab="MSE(beta)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

legend("bottomright",legend=c("Shrinkage","PreTest","W-AIC","BMA","MLE"),lwd=lthick,lty=ltype,bty="n")

windows()
par(mfrow=c(2,2))

rr=cbind(riskp[5,1,1,],riskp[2,1,1,],riskp[3,1,1,],riskp[4,1,1,],riskp[1,1,1,])
matplot(cc,rr,type="l",main=paste("n=",nlist[1],", p=",plist[1]),xlab="c",ylab="MSE(pi)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

rr=cbind(riskp[5,1,2,],riskp[2,1,2,],riskp[3,1,2,],riskp[4,1,2,],riskp[1,1,2,])
matplot(cc,rr,type="l",main=paste("n=",nlist[2],", p=",plist[1]),xlab="c",ylab="MSE(pi)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

rr=cbind(riskp[5,2,1,],riskp[2,2,1,],riskp[3,2,1,],riskp[4,2,1,],riskp[1,2,1,])
matplot(cc,rr,type="l",main=paste("n=",nlist[1],", p=",plist[2]),xlab="c",ylab="MSE(pi)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

rr=cbind(riskp[5,2,2,],riskp[2,2,2,],riskp[3,2,2,],riskp[4,2,2,],riskp[1,2,2,])
matplot(cc,rr,type="l",main=paste("n=",nlist[2],", p=",plist[2]),xlab="c",ylab="MSE(pi)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

legend("bottomright",legend=c("Shrinkage","PreTest","W-AIC","BMA","MLE"),lwd=lthick,lty=ltype,bty="n")

windows()
par(mfrow=c(2,2))

rr=cbind(riskb[5,1,1,],riskb[6,1,1,],riskb[7,1,1,],riskb[8,1,1,],riskb[1,1,1,])
matplot(cc,rr,type="l",main=paste("n=",nlist[1],", p=",plist[1]),xlab="c",ylab="MSE(beta)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

rr=cbind(riskb[5,1,2,],riskb[6,1,2,],riskb[7,1,2,],riskb[8,1,2,],riskb[1,1,2,])
matplot(cc,rr,type="l",main=paste("n=",nlist[2],", p=",plist[1]),xlab="c",ylab="MSE(beta)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

rr=cbind(riskb[5,2,1,],riskb[6,2,1,],riskb[7,2,1,],riskb[8,2,1,],riskb[1,2,1,])
matplot(cc,rr,type="l",main=paste("n=",nlist[1],", p=",plist[2]),xlab="c",ylab="MSE(beta)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

rr=cbind(riskb[5,2,2,],riskb[6,2,2,],riskb[7,2,2,],riskb[8,2,2,],riskb[1,2,2,])
matplot(cc,rr,type="l",main=paste("n=",nlist[2],", p=",plist[2]),xlab="c",ylab="MSE(beta)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

legend("bottomright",legend=c("Shrinkage1","Shrinkage2","Shrinkage3","Shrinkage4","MLE"),lwd=lthick,lty=ltype,bty="n")

windows()
par(mfrow=c(2,2))

rr=cbind(riskp[5,1,1,],riskp[6,1,1,],riskp[7,1,1,],riskp[8,1,1,],riskp[1,1,1,])
matplot(cc,rr,type="l",main=paste("n=",nlist[1],", p=",plist[1]),xlab="c",ylab="MSE(pi)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

rr=cbind(riskp[5,1,2,],riskp[6,1,2,],riskp[7,1,2,],riskp[8,1,2,],riskp[1,1,2,])
matplot(cc,rr,type="l",main=paste("n=",nlist[2],", p=",plist[1]),xlab="c",ylab="MSE(pi)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

rr=cbind(riskp[5,2,1,],riskp[6,2,1,],riskp[7,2,1,],riskp[8,2,1,],riskp[1,2,1,])
matplot(cc,rr,type="l",main=paste("n=",nlist[1],", p=",plist[2]),xlab="c",ylab="MSE(pi)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

rr=cbind(riskp[5,2,2,],riskp[6,2,2,],riskp[7,2,2,],riskp[8,2,2,],riskp[1,2,2,])
matplot(cc,rr,type="l",main=paste("n=",nlist[2],", p=",plist[2]),xlab="c",ylab="MSE(pi)",lty=ltype,lwd=lthick,col=c(1,1,1),ylim=c(0,1.2))

legend("bottomright",legend=c("Shrinkage1","Shrinkage2","Shrinkage3","Shrinkage4","MLE"),lwd=lthick,lty=ltype,bty="n")

